/**
 * Copyright (c) 2015-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule;

/**
 * @author Christophe DENEUX - Linagora
 */
public class DeployServiceAssemblyTaskTest {

    @Rule
    public final PetalsJmxApiJunitRule jmxApi = new PetalsJmxApiJunitRule();

    /**
     * <p>
     * Check the Ant task 'jbi-deploy-service-assembly' where:
     * </p>
     * <ol>
     * <li>the SA deployment succeeds at level of the implementation’s execution of the task,</li>
     * <li>but the SA deployment fails at level of interaction(s) with component,</li>
     * <li>the attribute <code>failOnError</code> is not set,</li>
     * <li>the attribute <code>failOnComponentError</code> is not set.</li>
     * </ol>
     * <p>
     * Expected result: the Ant task succeeds, no error is thrown.
     * </p>
     */
    @Test
    public void deploymentWithErrorAtComponentLevel_failOnErrorNotSet_failOnComponentErrorNotSet() throws IOException,
 URISyntaxException, NotRemoteJMXClientException {

        final URL saUrl = new URL("file:///tmp/sa-with-failing-su-deployment.zip");
        final String saDeploymentResult = new String(Files.readAllBytes(new File(Thread.currentThread()
                .getContextClassLoader().getResource("deployment-report-sa-succeeds-with-2-sus-ok-and-in-error.xml")
                .toURI()).toPath()));
        this.jmxApi.registerServiceAssemblyDeployment(saUrl, saDeploymentResult.toString());

        final DeployServiceAssemblyTask dsat = new DeployServiceAssemblyTask();
        dsat.setPort(String.valueOf(this.jmxApi.getPort()));
        dsat.setUsername(this.jmxApi.getUsername());
        dsat.setPassword(this.jmxApi.getPassword());
        dsat.setFile(saUrl.toString());
        dsat.setProject(new Project());

        dsat.execute();
    }

    /**
     * <p>
     * Check the Ant task 'jbi-deploy-service-assembly' where:
     * </p>
     * <ol>
     * <li>the SA deployment succeeds at level of the implementation’s execution of the task,</li>
     * <li>but the SA deployment fails at level of interaction(s) with component,</li>
     * <li>the attribute <code>failOnError</code> is set to <code>false</code>,</li>
     * <li>the attribute <code>failOnComponentError</code> is not set.</li>
     * </ol>
     * <p>
     * Expected result: the Ant task succeeds, no error is thrown.
     * </p>
     */
    @Test
    public void deploymentWithErrorAtComponentLevel_failOnErrorSetToFalse_failOnComponentErrorNotSet()
            throws IOException, URISyntaxException, NotRemoteJMXClientException {

        final URL saUrl = new URL("file:///tmp/sa-with-failling-su-deployment.zip");
        final String saDeploymentResult = new String(Files.readAllBytes(new File(Thread.currentThread()
                .getContextClassLoader().getResource("deployment-report-sa-succeeds-with-2-sus-ok-and-in-error.xml")
                .toURI()).toPath()));
        this.jmxApi.registerServiceAssemblyDeployment(saUrl, saDeploymentResult.toString());

        final DeployServiceAssemblyTask dsat = new DeployServiceAssemblyTask();
        dsat.setPort(String.valueOf(this.jmxApi.getPort()));
        dsat.setUsername(this.jmxApi.getUsername());
        dsat.setPassword(this.jmxApi.getPassword());
        dsat.setFile(saUrl.toString());
        dsat.setFailOnError(false);
        dsat.setProject(new Project());

        dsat.execute();
    }

    /**
     * <p>
     * Check the Ant task 'jbi-deploy-service-assembly' where:
     * </p>
     * <ol>
     * <li>the SA deployment succeeds at level of the implementation’s execution of the task,</li>
     * <li>but the SA deployment fails at level of interaction(s) with component,</li>
     * <li>the attribute <code>failOnError</code> is not set,</li>
     * <li>the attribute <code>failOnComponentError</code> is set to <code>true</code>.</li>
     * </ol>
     * <p>
     * Expected result: the Ant task fails.
     * </p>
     */
    @Test(expected = BuildException.class)
    public void deploymentWithErrorAtComponentLevel_failOnErrorNotSet_failOnComponentErrorSetToTrue()
            throws IOException, URISyntaxException, NotRemoteJMXClientException {

        final URL saUrl = new URL("file:///tmp/sa-with-failing-su-deployment.zip");
        final String saDeploymentResult = new String(Files.readAllBytes(new File(Thread.currentThread()
                .getContextClassLoader().getResource("deployment-report-sa-succeeds-with-2-sus-ok-and-in-error.xml")
                .toURI()).toPath()));
        this.jmxApi.registerServiceAssemblyDeployment(saUrl, saDeploymentResult.toString());

        final DeployServiceAssemblyTask dsat = new DeployServiceAssemblyTask();
        dsat.setPort(String.valueOf(this.jmxApi.getPort()));
        dsat.setUsername(this.jmxApi.getUsername());
        dsat.setPassword(this.jmxApi.getPassword());
        dsat.setFile(saUrl.toString());
        dsat.setFailOnComponentError(true);
        dsat.setProject(new Project());

        dsat.execute();
    }

    /**
     * <p>
     * Check the Ant task 'jbi-deploy-service-assembly' where:
     * </p>
     * <ol>
     * <li>the SA deployment succeeds at level of the implementation’s execution of the task,</li>
     * <li>but the SA deployment fails at level of interaction(s) with component,</li>
     * <li>the attribute <code>failOnError</code> is set to <code>false</code>,</li>
     * <li>the attribute <code>failOnComponentError</code> is set to <code>true</code>.</li>
     * </ol>
     * <p>
     * Expected result: the Ant task succeeds, no error is thrown.
     * </p>
     */
    @Test
    public void deploymentWithErrorAtComponentLevel_failOnErrorSetToFalse_failOnComponentErrorSetToTrue()
            throws IOException, URISyntaxException, NotRemoteJMXClientException {

        final URL saUrl = new URL("file:///tmp/sa-with-failing-su-deployment.zip");
        final String saDeploymentResult = new String(Files.readAllBytes(new File(Thread.currentThread()
                .getContextClassLoader().getResource("deployment-report-sa-succeeds-with-2-sus-ok-and-in-error.xml")
                .toURI()).toPath()));
        this.jmxApi.registerServiceAssemblyDeployment(saUrl, saDeploymentResult.toString());

        final DeployServiceAssemblyTask dsat = new DeployServiceAssemblyTask();
        dsat.setPort(String.valueOf(this.jmxApi.getPort()));
        dsat.setUsername(this.jmxApi.getUsername());
        dsat.setPassword(this.jmxApi.getPassword());
        dsat.setFile(saUrl.toString());
        dsat.setFailOnError(false);
        dsat.setFailOnComponentError(true);
        dsat.setProject(new Project());

        dsat.execute();
    }

    /**
     * <p>
     * Check the Ant task 'jbi-deploy-service-assembly' where:
     * </p>
     * <ol>
     * <li>the SA deployment fails at DeploymentServiceMBean level,</li>
     * <li>the attribute <code>failOnError</code> is not set,</li>
     * <li>the attribute <code>failOnComponentError</code> is not set.</li>
     * </ol>
     * <p>
     * Expected result: the Ant task fails.
     * </p>
     */
    @Test(expected = BuildException.class)
    public void deploymentWithErrorAtMBeanLevel_failOnErrorNotSet_failOnComponentErrorNotSet()
            throws MalformedURLException, NotRemoteJMXClientException {

        final URL saUrl = new URL("file:///tmp/sa-failing.zip");
        this.jmxApi.registerServiceAssemblyDeployment(saUrl, new DeploymentServiceErrorException(
                "Error during deployment"));

        final DeployServiceAssemblyTask dsat = new DeployServiceAssemblyTask();
        dsat.setPort(String.valueOf(this.jmxApi.getPort()));
        dsat.setUsername(this.jmxApi.getUsername());
        dsat.setPassword(this.jmxApi.getPassword());
        dsat.setFile(saUrl.toString());
        dsat.setProject(new Project());

        dsat.execute();
    }

    /**
     * <p>
     * Check the Ant task 'jbi-deploy-service-assembly' where:
     * </p>
     * <ol>
     * <li>the SA deployment fails at DeploymentServiceMBean level,</li>
     * <li>the attribute <code>failOnError</code> is set to <code>false</code>,</li>
     * <li>the attribute <code>failOnComponentError</code> is not set.</li>
     * </ol>
     * <p>
     * Expected result: the Ant task succeeds.
     * </p>
     */
    public void deploymentWithErrorAtMBeanLevel_failOnErrorSetToFalse_failOnComponentErrorNotSet()
            throws MalformedURLException, NotRemoteJMXClientException {

        final URL saUrl = new URL("file:///tmp/sa-failing.zip");
        this.jmxApi.registerServiceAssemblyDeployment(saUrl, new DeploymentServiceErrorException(
                "Error during deployment"));

        final DeployServiceAssemblyTask dsat = new DeployServiceAssemblyTask();
        dsat.setPort(String.valueOf(this.jmxApi.getPort()));
        dsat.setUsername(this.jmxApi.getUsername());
        dsat.setPassword(this.jmxApi.getPassword());
        dsat.setFile(saUrl.toString());
        dsat.setFailOnError(false);
        dsat.setFailOnComponentError(true);
        dsat.setProject(new Project());

        dsat.execute();
    }

    /**
     * <p>
     * Check the Ant task 'jbi-deploy-service-assembly' where:
     * </p>
     * <ol>
     * <li>the SA deployment succeeds completely,</li>
     * <li>the attribute <code>failOnComponentError</code> is not set.</li>
     * </ol>
     * <p>
     * Expected result: the Ant task succeeds without error.
     * </p>
     */
    public void deploymentWithoutError_failOnComponentErrorNotSet()
            throws IOException, URISyntaxException, NotRemoteJMXClientException {

        final URL saUrl = new URL("file:///tmp/sa-failing.zip");
        final String saDeploymentResult = new String(Files.readAllBytes(new File(Thread.currentThread()
                .getContextClassLoader().getResource("deployment-report-sa-succeeds.xml").toURI()).toPath()));
        this.jmxApi.registerServiceAssemblyDeployment(saUrl, saDeploymentResult.toString());

        final DeployServiceAssemblyTask dsat = new DeployServiceAssemblyTask();
        dsat.setPort(String.valueOf(this.jmxApi.getPort()));
        dsat.setUsername(this.jmxApi.getUsername());
        dsat.setPassword(this.jmxApi.getPassword());
        dsat.setFile(saUrl.toString());
        dsat.setProject(new Project());

        dsat.execute();
    }

    /**
     * <p>
     * Check the Ant task 'jbi-deploy-service-assembly' where:
     * </p>
     * <ol>
     * <li>the SA deployment succeeds completely,</li>
     * <li>the attribute <code>failOnComponentError</code> is set to <code>true</code>.</li>
     * </ol>
     * <p>
     * Expected result: the Ant task succeeds without error.
     * </p>
     */
    public void deploymentWithoutError_failOnComponentErrorSetToTrue()
            throws IOException, URISyntaxException, NotRemoteJMXClientException {

        final URL saUrl = new URL("file:///tmp/sa-failing.zip");
        final String saDeploymentResult = new String(Files.readAllBytes(new File(Thread.currentThread()
                .getContextClassLoader().getResource("deployment-report-sa-succeeds.xml").toURI()).toPath()));
        this.jmxApi.registerServiceAssemblyDeployment(saUrl, saDeploymentResult.toString());

        final DeployServiceAssemblyTask dsat = new DeployServiceAssemblyTask();
        dsat.setPort(String.valueOf(this.jmxApi.getPort()));
        dsat.setUsername(this.jmxApi.getUsername());
        dsat.setPassword(this.jmxApi.getPassword());
        dsat.setFile(saUrl.toString());
        dsat.setFailOnComponentError(true);
        dsat.setProject(new Project());

        dsat.execute();
    }
}
