/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.util.ReaderInputStream;
import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule;
import org.ow2.petals.jmx.api.mock.junit.SharedLibrary;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class UninstallSharedLibraryTaskTest {

    @Rule
    public final PetalsJmxApiJunitRule jmxApi = new PetalsJmxApiJunitRule();

    /**
     * Check {@link UninstallSharedLibraryTask} against a shared library name set to <code>null</code>.
     */
    @Test
    public void testSharedLibraryName_Null() {
        final UninstallSharedLibraryTask task = new UninstallSharedLibraryTask();
        task.setName(null);
        task.setVersion("version");

        try {
            task.execute();
            fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            assertNotNull(e.getMessage());
            assertTrue(e.getMessage().contains("Missing attribute 'name'"));
        }
    }

    /**
     * Check {@link UninstallSharedLibraryTask} against an empty shared library name.
     */

    @Test
    public void testSharedLibraryName_Empty() {
        final UninstallSharedLibraryTask task = new UninstallSharedLibraryTask();
        task.setName("");
        task.setVersion("version");

        try {
            task.execute();
            fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            assertNotNull(e.getMessage());
            assertTrue(e.getMessage().contains("Empty attribute 'name'"));
        }
    }

    /**
     * Check {@link UninstallSharedLibraryTask} against a shared library version set to <code>null</code>.
     */
    @Test
    public void testSharedLibraryVersion_Null() {
        final UninstallSharedLibraryTask task = new UninstallSharedLibraryTask();
        task.setName("name");
        task.setVersion(null);

        try {
            task.execute();
            fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            assertNotNull(e.getMessage());
            assertTrue(e.getMessage().contains("Missing attribute 'version'"));
        }
    }

    /**
     * Check {@link UninstallSharedLibraryTask} against an empty shared library version.
     */
    @Test
    public void testSharedLibraryVersion_Empty() {
        final UninstallSharedLibraryTask task = new UninstallSharedLibraryTask();
        task.setName("name");
        task.setVersion("");

        try {
            task.execute();
            fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            assertNotNull(e.getMessage());
            assertTrue(e.getMessage().contains("Empty attribute 'version'"));
        }
    }

    /**
     * Check {@link UninstallSharedLibraryTask} against only the shared library name is set and version is missing.
     */
    @Test
    public void testSharedLibraryVersion_Missing() {
        final UninstallSharedLibraryTask task = new UninstallSharedLibraryTask();
        task.setName("name");

        try {
            task.execute();
            fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            assertNotNull(e.getMessage());
            assertTrue(e.getMessage().contains("Missing attribute 'version'"));
        }
    }

    /**
     * Check {@link UninstallSharedLibraryTask} against only the shared library version is set and name is missing.
     */
    @Test
    public void testSharedLibraryName_Missing() {
        final UninstallSharedLibraryTask task = new UninstallSharedLibraryTask();
        task.setVersion("version");

        try {
            task.execute();
            fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            assertNotNull(e.getMessage());
            assertTrue(e.getMessage().contains("Missing attribute 'name'"));
        }
    }

    /**
     * <p>
     * Check the Ant task 'jbi-uninstall-shared-library'
     * </p>
     * <p>
     * Environment:
     * </p>
     * <ul>
     * <li>3 SLs installed: SL1, SL2, SL3. SL1 and SL2 have the same version</li>
     * </ul>
     * <p>
     * Expected result: no error occurs, only the expected shared library is uninstalled.
     * </p>
     */
    @Test
    public void uninstallSharedLibrary() throws AdminServiceErrorException, ComponentErrorException,
            InstallationServiceErrorException, SAXException, IOException, NotRemoteJMXClientException {

        final String slName1 = "sl-used-1";
        final String slVersion1 = "version-1";
        final String slName2 = "sl-used-2";
        final String slVersion2 = "version-2";
        final String resultProperty = "result-property";

        // Prepare the JBI environment mocked

        // SL installed
        final SharedLibrary sl1 = new SharedLibrary(slName1, slVersion1);
        this.jmxApi.addSharedLibrary(sl1);
        final SharedLibrary sl2 = new SharedLibrary(slName2, slVersion1);
        this.jmxApi.addSharedLibrary(sl2);
        final SharedLibrary sl3 = new SharedLibrary(slName2, slVersion2);
        this.jmxApi.addSharedLibrary(sl3);

        final UninstallSharedLibraryTask task = new UninstallSharedLibraryTask();
        task.setName(slName2);
        task.setVersion(slVersion1);
        task.setPort(String.valueOf(this.jmxApi.getPort()));
        task.setUsername(this.jmxApi.getUsername());
        task.setPassword(this.jmxApi.getPassword());

        task.execute();

        // Check that the expected SL was uninstalled
        final ListSharedLibrariesTask lset = new ListSharedLibrariesTask();
        lset.setPort(String.valueOf(this.jmxApi.getPort()));
        lset.setUsername(this.jmxApi.getUsername());
        lset.setPassword(this.jmxApi.getPassword());
        lset.setXmlOutput(resultProperty);
        lset.setProject(new Project());

        lset.execute();
        final String result = lset.getProject().getProperty(resultProperty);

        final DocumentBuilder docBuilder = DocumentBuilders.takeDocumentBuilder();
        final Reader resultReader = new StringReader(result);
        final InputStream resultInputStream = new ReaderInputStream(resultReader);
        final Document resultDoc = docBuilder.parse(resultInputStream);
        final Element sharedLibraryInfoListElement = resultDoc.getDocumentElement();
        assertEquals("http://java.sun.com/xml/ns/jbi/shared-library-info-list",
                sharedLibraryInfoListElement.getNamespaceURI());
        assertEquals("shared-library-info-list", sharedLibraryInfoListElement.getNodeName());
        assertEquals("1.0", sharedLibraryInfoListElement.getAttribute("version"));
        final NodeList sharedLibraryInfos = sharedLibraryInfoListElement.getElementsByTagNameNS(
                "http://java.sun.com/xml/ns/jbi/shared-library-info-list", "shared-library-info");
        assertNotNull(sharedLibraryInfos);
        assertEquals(2, sharedLibraryInfos.getLength());
        boolean isSlFound1 = false;
        boolean isSlFound2 = false;
        for (int i = 0; i < 2; i++) {
            final Node sharedLibraryInfo = sharedLibraryInfos.item(i);
            assertTrue(sharedLibraryInfo instanceof Element);
            final Element sharedLibraryInfoElt = (Element) sharedLibraryInfo;
            final String slName = sharedLibraryInfoElt.getAttribute("name");
            if (slName.equals(slName1)) {
                assertEquals(slVersion1, sharedLibraryInfoElt.getAttribute("version"));
                isSlFound1 = true;
            } else if (slName.equals(slName2)) {
                assertEquals(slVersion2, sharedLibraryInfoElt.getAttribute("version"));
                isSlFound2 = true;
            } else {
                fail("Unexpected SL name");
            }
        }
        assertTrue(slName1 + " not found", isSlFound1);
        assertTrue(slName2 + " not found", isSlFound2);
    }
}
