/**
 * Copyright (c) 2015-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Set;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.types.DirSet;
import org.junit.Test;
import org.ow2.petals.ant.task.monit.exception.BuildDirChooseDirSpecificationException;
import org.ow2.petals.ant.task.monit.exception.BuildDirEmptyException;
import org.ow2.petals.ant.task.monit.exception.BuildDirMissingException;
import org.ow2.petals.ant.task.monit.exception.BuildIdEmptyException;
import org.ow2.petals.ant.task.monit.exception.BuildIdMissingException;
import org.ow2.petals.log.parser.api.exception.FlowStepFieldInvalidOrMissingException;
import org.ow2.petals.log.parser.api.exception.FlowStepFieldRequiredMissingException;

public class ReadLogFilesTaskTest extends AbstractMonitTraceTest {

    /**
     * Check the loading of monit trace on nominal conditions.
     */
    @Test
    public void nominal() throws URISyntaxException {

        // ------------------------
        // using attribute 'dir'
        this.loadMonitTraces();

        // ------------------------
        // using nested element 'dirset'
        final Project project = new Project();
        final URL petalsRootLogFile = Thread.currentThread().getContextClassLoader().getResource("monit");
        assertNotNull("Petals root log file directory not found", petalsRootLogFile);

        final ReadLogFilesTask rlfTask = new ReadLogFilesTask();
        final DirSet dirSet = new DirSet();
        dirSet.setDir(new File(petalsRootLogFile.toURI()));
        dirSet.setIncludes("**/flow-monitoring");
        rlfTask.addDirSet(dirSet);
        rlfTask.setId(REF_ID);
        rlfTask.setProject(project);

        rlfTask.execute();

        final Object monitTracesObj = project.getReference(REF_ID);
        assertTrue(monitTracesObj instanceof Set);
        assertEquals(7, ((Set<?>) monitTracesObj).size());
    }

    /**
     * <p>
     * Check the loading of MONIT traces containing a missing trace code setting <code>failOnError</code> to
     * <code>true</code> or <code>false</code>.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>if <code>failOnError</code> is set to <code>true</code>, the error
     * {@link FlowStepFieldInvalidOrMissingException} occurs because the trace code is missing,</li>
     * <li>if <code>failOnError</code> is set to <code>false</code>, no error is raised and MONIT traces are loaded.
     * </li>
     * </ul>
     */
    @Test
    public void fieldTraceCodeMissing() throws URISyntaxException {

        this.fieldErrors("monit/flow-monitoring-missing-traceCode", Integer.valueOf(1),
                FlowStepFieldInvalidOrMissingException.class);
    }

    private void fieldErrors(final String resourceLog, final Integer nbTracesLoaded, final Class<?> expectedError)
            throws URISyntaxException {

        final Project project = new Project();
        final URL petalsLogFile = Thread.currentThread().getContextClassLoader().getResource(resourceLog);
        assertNotNull("Petals log file not found", petalsLogFile);

        final ReadLogFilesTask rlfTask = new ReadLogFilesTask();
        rlfTask.setDir(new File(petalsLogFile.toURI()).getAbsolutePath());
        rlfTask.setId(REF_ID);
        rlfTask.setProject(project);

        // failOnError set to false
        rlfTask.setFailOnError(false);
        rlfTask.execute();
        final Object monitTracesObj = project.getReference(REF_ID);
        if (nbTracesLoaded == null) {
            assertNull(monitTracesObj);
        } else {
            assertNotNull(monitTracesObj);
            assertTrue(monitTracesObj instanceof Set);
            assertEquals(nbTracesLoaded.intValue(), ((Set<?>) monitTracesObj).size());
        }

        // failOnError set to true
        try {
            rlfTask.setFailOnError(true);
            rlfTask.execute();
            fail("The ANT task should fail because a flow step field is missing or invalid on a MONIT trace");
        } catch (final BuildException e) {
            // Expected exception
            assertTrue(expectedError.isInstance(e.getCause()));
        }
    }

    /**
     * <p>
     * Check the loading of MONIT traces containing an invalid trace code setting <code>failOnError</code> to
     * <code>true</code> or <code>false</code>.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>if <code>failOnError</code> is set to <code>true</code>, the error
     * {@link FlowStepFieldInvalidOrMissingException} occurs because the trace code is invalid,</li>
     * <li>if <code>failOnError</code> is set to <code>false</code>, no error is raised and MONIT traces are loaded.
     * </li>
     * </ul>
     */
    @Test
    public void fieldTraceCodeInvalid() throws URISyntaxException {

        this.fieldErrors("monit/flow-monitoring-invalid-traceCode", Integer.valueOf(1),
                FlowStepFieldInvalidOrMissingException.class);
    }

    /**
     * <p>
     * Check the loading of MONIT traces containing a missing flow instance identifier setting <code>failOnError</code>
     * to <code>true</code> or <code>false</code>.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>if <code>failOnError</code> is set to <code>true</code>, the error
     * {@link FlowStepFieldInvalidOrMissingException} occurs because the flow instance identifier is missing,</li>
     * <li>if <code>failOnError</code> is set to <code>false</code>, no error is raised and MONIT traces are loaded.
     * </li>
     * </ul>
     */
    @Test
    public void fieldFlowInstanceIdMissing() throws URISyntaxException {

        this.fieldErrors("monit/flow-monitoring-missing-flowInstanceId", Integer.valueOf(1),
                FlowStepFieldInvalidOrMissingException.class);
    }

    /**
     * <p>
     * Check the loading of MONIT traces containing an invalid flow instance identifier setting <code>failOnError</code>
     * to <code>true</code> or <code>false</code>.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>if <code>failOnError</code> is set to <code>true</code>, the error
     * {@link FlowStepFieldInvalidOrMissingException} occurs because the flow instance identifier is invalid,</li>
     * <li>if <code>failOnError</code> is set to <code>false</code>, no error is raised and MONIT traces are loaded.
     * </li>
     * </ul>
     */
    @Test
    public void fieldFlowInstanceIdInvalid() throws URISyntaxException {

        this.fieldErrors("monit/flow-monitoring-invalid-flowInstanceId", Integer.valueOf(1),
                FlowStepFieldInvalidOrMissingException.class);
    }

    /**
     * <p>
     * Check the loading of MONIT traces containing a missing flow step identifier setting <code>failOnError</code> to
     * <code>true</code> or <code>false</code>.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>if <code>failOnError</code> is set to <code>true</code>, the error
     * {@link FlowStepFieldRequiredMissingException} occurs because the flow step identifier is a required field,</li>
     * <li>if <code>failOnError</code> is set to <code>false</code>, no error is raised and no MONIT traces is loaded.
     * </li>
     * </ul>
     */
    @Test
    public void fieldFlowStepIdMissing() throws Throwable {

        this.fieldErrors("monit/flow-monitoring-missing-flowStepId", null, FlowStepFieldRequiredMissingException.class);
    }

    /**
     * <p>
     * Check the loading of MONIT traces containing an invalid flow step identifier setting <code>failOnError</code> to
     * <code>true</code> or <code>false</code>.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>if <code>failOnError</code> is set to <code>true</code>, the error
     * {@link FlowStepFieldInvalidOrMissingException} occurs because the flow step identifier is invalid,</li>
     * <li>if <code>failOnError</code> is set to <code>false</code>, no error is raised and no MONIT traces is loaded.
     * </li>
     * </ul>
     */
    @Test
    public void fieldFlowStepIdInvalid() throws Throwable {

        this.fieldErrors("monit/flow-monitoring-invalid-flowStepId", Integer.valueOf(0),
                FlowStepFieldInvalidOrMissingException.class);
    }

    /**
     * Check the processing of Ant task 'petals-monit-read' when the parameter 'id' is missing.
     */
    @Test(expected = BuildIdMissingException.class)
    public void noId() throws URISyntaxException {

        final ReadLogFilesTask rlfTask = new ReadLogFilesTask();

        rlfTask.execute();
    }

    /**
     * Check the processing of Ant task 'petals-monit-read' when the parameter 'id' is empty.
     */
    @Test(expected = BuildIdEmptyException.class)
    public void emptyId() throws URISyntaxException {

        final ReadLogFilesTask rlfTask = new ReadLogFilesTask();
        rlfTask.setId("");

        rlfTask.execute();
    }

    /**
     * Check the processing of Ant task 'petals-monit-read' when the parameter 'dir' is missing and no nested element
     * 'dirset' is given.
     */
    @Test(expected = BuildDirMissingException.class)
    public void noDirAndNoDirSet() throws URISyntaxException {

        final ReadLogFilesTask rlfTask = new ReadLogFilesTask();
        rlfTask.setId("a-value");

        rlfTask.execute();
    }

    /**
     * Check the processing of Ant task 'petals-monit-read' when the parameter 'dir' is given with nested element
     * 'dirset'.
     */
    @Test(expected = BuildDirChooseDirSpecificationException.class)
    public void dirAndDirSet() throws URISyntaxException {

        final URL petalsRootLogFile = Thread.currentThread().getContextClassLoader().getResource("monit");
        assertNotNull("Petals root log file directory not found", petalsRootLogFile);

        final Project project = new Project();
        final ReadLogFilesTask rlfTask = new ReadLogFilesTask();
        rlfTask.setDir(new File(new File(petalsRootLogFile.toURI()), "flow-monitoring").getAbsolutePath());
        final DirSet dirSet = new DirSet();
        dirSet.setDir(new File(petalsRootLogFile.toURI()));
        dirSet.setIncludes("**/flow-monitoring");
        rlfTask.addDirSet(dirSet);
        rlfTask.setId(REF_ID);
        rlfTask.setProject(project);

        rlfTask.execute();
    }

    /**
     * Check the processing of Ant task 'petals-monit-read' when the parameter 'dir' is empty.
     */
    @Test(expected = BuildDirEmptyException.class)
    public void emptyDir() throws URISyntaxException {

        final ReadLogFilesTask rlfTask = new ReadLogFilesTask();
        rlfTask.setId("a-value");
        rlfTask.setDir("");

        rlfTask.execute();
    }

    /**
     * Check the processing of Ant task 'petals-monit-read' when the parameter 'dir' does not exist.
     */
    @Test(expected = BuildException.class)
    public void notExistDir() throws URISyntaxException {

        final ReadLogFilesTask rlfTask = new ReadLogFilesTask();
        rlfTask.setId("a-value");
        rlfTask.setDir("unexisting-directory");

        rlfTask.execute();
    }

}
