/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit.assertion.flowtree;

import java.net.URISyntaxException;

import org.apache.tools.ant.Project;
import org.junit.Test;
import org.ow2.petals.ant.task.monit.AbstractMonitTraceTest;
import org.ow2.petals.ant.task.monit.assertion.exception.BuildAssertParamInvalidValueException;
import org.ow2.petals.ant.task.monit.assertion.exception.BuildAssertParamMissingException;
import org.ow2.petals.ant.task.monit.assertion.flowtree.exception.BuildAssertFlowTreeNodeNotFoundException;
import org.ow2.petals.ant.task.monit.assertion.flowtreenode.AssertFlowTreeNode;
import org.ow2.petals.ant.task.monit.assertion.flowtreenode.FlowInstanceIdEquals;
import org.ow2.petals.ant.task.monit.assertion.flowtreenode.exception.BuildFlowInstanceIdInvalidException;

/**
 * Unit tests of {@link AssertFlowTreeTask}. Unit tests of nested assertions are in package
 * <code>org.ow2.petals.ant.task.monit.assertion.flowtreenode</code>.
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class AssertFlowTreeTaskTest extends AbstractMonitTraceTest {

    private static final String BC_GATEWAY_ROOT_FLOW_ID = "2c28f3c0-021c-11e6-8bbf-024261b18bb0";

    private static final String BC_GATEWAY_CHILD_FLOW_ID = "2c2e4af0-021c-11e6-b094-024261b18bb0";

    private static final String BC_GATEWAY_SUB_CHILD_FLOW_ID = "2c337b10-021c-11e6-85ed-024261b18bb0";

    private static final String SE_ACTIVITI_ROOT_FLOW_ID_1 = "15f132a0-0575-11e6-83d8-024282c01af2";

    private static final String SE_ACTIVITI_ROOT_FLOW_ID_2 = "1a319b70-0575-11e6-83d8-024282c01af2";

    private static final String SE_ACTIVITI_PROCESS_FLOW_ID = "160eccc0-0575-11e6-83d8-024282c01af2";

    /**
     * Check assertions on flow tree nodes of a given flow tree read from BC Gateway MONIT traces under nominal
     * conditions
     */
    @Test
    public void nominalBcGwRootFlowTreeNode() throws URISyntaxException {

        final Project project = this.buildFlowTrees();

        final FlowInstanceIdEquals rootFlowInstanceIdEquals = new FlowInstanceIdEquals();
        rootFlowInstanceIdEquals.setExpected(BC_GATEWAY_ROOT_FLOW_ID);
        final AssertFlowTreeNode rootTreeNodeAssert = new AssertFlowTreeNode();
        rootTreeNodeAssert.setPath("");
        rootTreeNodeAssert.addFlowInstanceIdEquals(rootFlowInstanceIdEquals);

        final FlowInstanceIdEquals childFlowInstanceIdEquals = new FlowInstanceIdEquals();
        childFlowInstanceIdEquals.setExpected(BC_GATEWAY_CHILD_FLOW_ID);
        final AssertFlowTreeNode childTreeNodeAssert = new AssertFlowTreeNode();
        childTreeNodeAssert.setPath("0");
        childTreeNodeAssert.addFlowInstanceIdEquals(childFlowInstanceIdEquals);

        final FlowInstanceIdEquals subChildFlowInstanceIdEquals = new FlowInstanceIdEquals();
        subChildFlowInstanceIdEquals.setExpected(BC_GATEWAY_SUB_CHILD_FLOW_ID);
        final AssertFlowTreeNode subChildTreeNodeAssert = new AssertFlowTreeNode();
        subChildTreeNodeAssert.setPath("0.0");
        subChildTreeNodeAssert.addFlowInstanceIdEquals(subChildFlowInstanceIdEquals);

        final AssertFlowTreeTask task = new AssertFlowTreeTask();
        task.setRefid(REF_TREE_ID);
        task.setFlowInstanceId(BC_GATEWAY_ROOT_FLOW_ID);
        task.addAssertFlowTreeNode(rootTreeNodeAssert);
        task.addAssertFlowTreeNode(childTreeNodeAssert);
        task.addAssertFlowTreeNode(subChildTreeNodeAssert);
        task.setProject(project);

        task.execute();
    }

    /**
     * Check assertions on flow tree nodes of a given flow tree read from SE Activiti MONIT traces under nominal
     * conditions
     */
    @Test
    public void nominalSeActivitiRootFlowTreeNode() throws URISyntaxException {

        final Project project = this.buildFlowTrees();

        final FlowInstanceIdEquals processFlowInstanceIdEquals = new FlowInstanceIdEquals();
        processFlowInstanceIdEquals.setExpected(SE_ACTIVITI_PROCESS_FLOW_ID);
        final AssertFlowTreeNode processTreeNodeAssert = new AssertFlowTreeNode();
        processTreeNodeAssert.setPath("0");
        processTreeNodeAssert.addFlowInstanceIdEquals(processFlowInstanceIdEquals);

        final AssertFlowTreeTask task = new AssertFlowTreeTask();
        task.setRefid(REF_TREE_ID);
        task.setFlowInstanceId(SE_ACTIVITI_ROOT_FLOW_ID_1);
        task.addAssertFlowTreeNode(processTreeNodeAssert);
        task.setProject(project);

        task.execute();

        task.setFlowInstanceId(SE_ACTIVITI_ROOT_FLOW_ID_2);
        task.execute();
    }

    /**
     * Check assertions on flow tree node of a given flow tree identified by a flow instance that does not exist
     */
    @Test(expected = BuildFlowInstanceIdInvalidException.class)
    public void flowInstanceIdNotFound() throws URISyntaxException {

        final Project project = this.buildFlowTrees();

        final AssertFlowTreeTask task = new AssertFlowTreeTask();
        task.setRefid(REF_TREE_ID);
        task.setFlowInstanceId("unknown-flow-instance-id");
        task.setProject(project);

        task.execute();
    }

    /**
     * Check assertions on flow tree node of a given flow tree identified by a flow instance, where the path identifying
     * the flow tree node is {@code null}.
     */
    @Test(expected = BuildAssertParamMissingException.class)
    public void pathNull() throws URISyntaxException {

        final Project project = this.buildFlowTrees();

        final AssertFlowTreeNode treeNodeAssert = new AssertFlowTreeNode();
        treeNodeAssert.setPath(null);

        final AssertFlowTreeTask task = new AssertFlowTreeTask();
        task.setRefid(REF_TREE_ID);
        task.addAssertFlowTreeNode(treeNodeAssert);
        task.setFlowInstanceId(BC_GATEWAY_ROOT_FLOW_ID);
        task.setProject(project);

        task.execute();
    }

    /**
     * Check assertions on flow tree node of a given flow tree identified by a flow instance, where the path identifying
     * the flow tree node is invalid.
     */
    @Test(expected = BuildAssertParamInvalidValueException.class)
    public void pathInvalid() throws URISyntaxException {

        final Project project = this.buildFlowTrees();

        final AssertFlowTreeNode treeNodeAssert = new AssertFlowTreeNode();
        treeNodeAssert.setPath("path.invalid");

        final AssertFlowTreeTask task = new AssertFlowTreeTask();
        task.setRefid(REF_TREE_ID);
        task.addAssertFlowTreeNode(treeNodeAssert);
        task.setFlowInstanceId(BC_GATEWAY_ROOT_FLOW_ID);
        task.setProject(project);

        task.execute();
    }

    /**
     * Check assertions on flow tree node of a given flow tree identified by a flow instance, where the path identifying
     * the flow tree node points to a not found flow tree node.
     */
    @Test(expected = BuildAssertFlowTreeNodeNotFoundException.class)
    public void flowTreeNodeNotFound() throws URISyntaxException {

        final Project project = this.buildFlowTrees();

        final AssertFlowTreeNode treeNodeAssert = new AssertFlowTreeNode();
        treeNodeAssert.setPath("10.20");

        final AssertFlowTreeTask task = new AssertFlowTreeTask();
        task.setRefid(REF_TREE_ID);
        task.addAssertFlowTreeNode(treeNodeAssert);
        task.setFlowInstanceId(BC_GATEWAY_ROOT_FLOW_ID);
        task.setProject(project);

        task.execute();
    }
}
